import os

from dotenv import load_dotenv
from openai import OpenAI


class OpenAISettings:
    def __init__(self):
        load_dotenv()
        self.client = OpenAI(api_key=os.getenv('OPENAI_API_KEY'))
        self.model = 'gpt-3.5-turbo'
        self.temperature = 0.1
        self.max_tokens = 2000

    def prompt(self, system, prompt):
        messages = [
            {"role": "system", "content": system},
            {"role": "user", "content": prompt}
        ]
        response = self.client.chat.completions.create(
            model=self.model,
            messages=messages,
        )
        return response.choices[0].message.content



import os

from dotenv import load_dotenv
from rich import print
from rich.panel import Panel
from rich import print
import typer
import typer.main

from .Assist.DjangoApplication import DjangoApplication


# convert to CLI and to be dynamic
# 1. Scan for Django apps
# 2. Keep the correct app_name in memory
# 3. Keep the correct app_directory in memory
# 4. ask the user to pick the brief file (is it readme or is it another file?)
# 5. Read the brief file, keep it in memory
# 6. Ask the user if they want to generate a better brief
# 7. if yes call generate with better_brief=True
class AssistedDjangoCLI:
    def __init__(self):

        print(Panel(renderable="CLI Tool to generate code for a singular django app"
                               "with the help of GPT3/4", highlight=True,
                    border_style="bold green", title="ASSISTED DJANGO",
                    subtitle="by @fauzaanu | https://t.me/fauzaanu"))

        # # if env is not set
        # NOT WORKING AS EXPECTED TODO: FIX
        # load_dotenv()
        # if not os.getenv("OPENAI_API_KEY"):
        #     print("Please set OPENAI_API_KEY in your .env file")
        #     raise typer.Exit()

        self.scan_for_apps()

    def scan_for_apps(self):
        """
        Scans for Django apps in the current directory.
        """

        if not os.path.exists("manage.py"):
            # typer.echo("Please run this command from the root of your Django project.")
            print("Please run this command from the root of your Django project. :frowning:")
            raise typer.Exit()

        django_apps = []
        required_files = ["__init__.py", "models.py"]

        # grab all the directories in the current directory
        directories = next(os.walk("."))[1]
        # print(directories)
        for directory in directories:
            # check if the directory has the required files
            if all(file in os.listdir(directory) for file in required_files):
                django_apps.append(directory)

        if django_apps:
            # Map the apps to numbers
            app_map = {}
            for index, app in enumerate(django_apps):
                app_map[index] = app

            # Print the apps with their full paths
            print("[bold green]Django Apps found[/bold green]")
            for index, app in app_map.items():
                app_path = os.path.abspath(app)
                typer.echo(
                    typer.style(f"{index + 1}. {app} - {app_path}", fg=typer.colors.BRIGHT_RED))

            # Ask the user to pick an app
            app_index = typer.prompt("\nWhich app would you like assistance with?", type=int)

            app_name = app_map[app_index - 1]
            appdir = os.path.abspath(app_name)

            print(f"[bold blue]App name[/bold blue] [yellow]{app_name}[/yellow]")
            print(f"[bold blue]App directory[/bold blue] [yellow]{appdir}[/yellow]")

            def check_for_briefs(directory=appdir):
                brief_files = []

                for item in os.listdir(directory):
                    if os.path.isdir(item):
                        continue
                    elif item.endswith(".md"):
                        # check if there is at least 1 line in the file
                        fullpath = os.path.join(directory, item)
                        with open(fullpath, "r") as f:
                            lines = f.readlines()
                            if len(lines) > 0:
                                brief_files.append((item, fullpath))

                return progress_brief_file(brief_files, appdir)

            def progress_brief_file(brief_files, appdir):
                if brief_files:
                    print("\n[bold green]Briefs found for[/bold green] [yellow]{}[/yellow]".format(app_name))
                    for index, (file, fullpath) in enumerate(brief_files):
                        typer.echo(typer.style(f"{index + 1}. {file} - {fullpath}",
                                               fg=typer.colors.BRIGHT_RED))

                    # ask the user to pick a brief file
                    brief_index = typer.prompt("\nWhich brief file would you like to use?", type=int)
                    brief_file, brief_file_path = brief_files[brief_index - 1]

                    # make sure brief_file_path is not None
                    if brief_file_path:
                        return brief_file_path
                    else:
                        print("Brief file path is None")
                        # exit the program
                        raise typer.Exit()
                else:
                    print("[bold red]No brief files found[/bold red] :cry:")
                    typer.confirm(
                        typer.style("Please add a brief file to the app directory and confirm", fg=typer.colors.RED), )
                    return None

            brief_file_path = check_for_briefs()
            # ask the user if they want to generate a better brief
            # print("Great briefs include :flexed_biceps: \n- database design ideas \n- functionality \n- purpose "
            #       "of the application")
            print("\n:flexed_biceps: [bold blue]Great briefs include[/bold blue] \n- database design ideas \n- "
                  "functionality \n- purpose of the application")
            generate_better_brief = typer.confirm(
                typer.style(f"Generate a better brief? (DOES NOT OVERWRITE EXISTING BREIF)",
                            bg=typer.colors.RED))

            # check if the brief file has atleast 1 line
            with open(brief_file_path, "r") as f:
                lines = f.readlines()
                if len(lines) == 0:
                    # typer.echo(typer.style("Brief file is empty", fg=typer.colors.RED))
                    # print("Brief file is empty :cry:")
                    print("[bold red]Brief file is empty[/bold red] :cry:")
                    raise typer.Exit()

            with open(brief_file_path, "r") as f:
                purpose = f.read()
            django_app = DjangoApplication(app_name, purpose, appdir)
            django_app.generate(better_brief=generate_better_brief)

            print("[bold green]Congrats. Generation is complete! :tada:[/bold green]")
            print("[bold blue]Made by @fauzaanu[/bold blue]")
            print("[bold red]:red_envelope: https://t.me/fauzaanu[/bold red]")
            print("[bold green]Happy Coding! :smiley:[/bold green]")
            print(
                "[blue]'Forget about trying, because if your just trying failing is still an option.'[/blue][gray] - "
                "Tim Grover[/gray]")
        else:
            # typer.echo("No Django Apps found.")
            # print("No Django Apps found. :cry:")
            print("[bold red]No Django Apps found.[/bold red] :cry:")


if __name__ == "__main__":
    typer.run(AssistedDjangoCLI)



ADMIN_EXAMPLE = """from django.contrib import admin
from .models import Post, Category

class PostAdmin(admin.ModelAdmin):
    list_display = ['title', 'created_on', 'author', 'category']


class CategoryAdmin(admin.ModelAdmin):
    list_display = ['name']

admin.site.register(Post, PostAdmin)
admin.site.register(Category, CategoryAdmin)      
  
# NOTES TO DEVELOPER:       
# [NOTE]       
# [NOTE]   
# [NOTE]   
# [NOTE]    
"""

FORMS_EXAMPLE = """from django import forms
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout, Submit
from .models import Users, Posts, Comments, Replies

class UsersForm(forms.ModelForm):
    class Meta:
        model = Users
        fields = ['name', 'email', 'password']

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_method = 'post'
        self.helper.add_input(Submit('submit', 'Submit'))

class PostsForm(forms.ModelForm):
    class Meta:
        model = Posts
        fields = ['user_id', 'post_text']

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_method = 'post'
        self.helper.add_input(Submit('submit', 'Submit'))

class CommentsForm(forms.ModelForm):
    class Meta:
        model = Comments
        fields = ['user_id', 'comment_text']

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_method = 'post'
        self.helper.add_input(Submit('submit', 'Submit'))

class RepliesForm(forms.ModelForm):
    class Meta:
        model = Replies
        fields = ['user_id', 'reply_text']

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_method = 'post'
        self.helper.add_input(Submit('submit', 'Submit'))
 
             
# NOTES TO DEVELOPER:       
# You will need to install crispy_forms in your virtual environment.
# [NOTE]   
# [NOTE]   
# [NOTE]    
"""

TESTS_EXAMPLE = """from django.test import TestCase
from .models import Post, Category

class PostModelTest(TestCase):
    @classmethod
    def setUpTestData(cls):
        # Set up non-modified objects used by all test methods
        Category.objects.create(name='Sample Category')
        Post.objects.create(title='Test title', content='Test content', category=Category.objects.get(id=1))

    def test_title_label(self):
        post = Post.objects.get(id=1)
        field_label = post._meta.get_field('title').verbose_name
        self.assertEqual(field_label, 'title')

    def test_content_is_required(self):
        post = Post.objects.get(id=1)
        field = post._meta.get_field('content')
        self.assertFalse(field.null)

class CategoryModelTest(TestCase):
    @classmethod
    def setUpTestData(cls):
        Category.objects.create(name='Sample Category')

    def test_name_max_length(self):
        category = Category.objects.get(id=1)
        max_length = category._meta.get_field('name').max_length
        self.assertEqual(max_length, 200)
        
        
# NOTES TO DEVELOPER:       
# [NOTE]       
# [NOTE]   
# [NOTE]   
# [NOTE]    
"""

URLS_EXAMPLE = """from django.urls import path
from . import views

app_name = 'blog'  # application namespace
urlpatterns = [
    path('', views.PostListView.as_view(), name='home'),
    path('post/<int:pk>/', views.PostDetailView.as_view(), name='post_detail'),
    path('category/<int:pk>/', views.CategoryListView.as_view(), name='category_list'),
]   
     
# NOTES TO DEVELOPER:       
# [NOTE]       
# [NOTE]   
# [NOTE]   
# [NOTE]    
"""

VIEWS_EXAMPLE = """from django.views import generic
from .models import Post, Category

class PostListView(generic.ListView):
    model = Post
    template_name = 'post_list.html'  # your template name
    context_object_name = 'posts'  # your variable name in the template

class PostDetailView(generic.DetailView):
    model = Post
    template_name = 'post_detail.html'  # your template name

class CategoryListView(generic.ListView):
    model = Category
    template_name = 'category_list.html'  # your template name
    context_object_name = 'categories'  # your variable name in the template
        
# NOTES TO DEVELOPER:       
# [NOTE]       
# [NOTE]   
# [NOTE]   
# [NOTE]    
"""

MODEL_EXAMPLE = """from django.db import models
from django.contrib.auth.models import User

class Category(models.Model):
    name = models.CharField(max_length=200, unique=True)

    def __str__(self):
        return self.name


class Post(models.Model):
    title = models.CharField(max_length=200)
    content = models.TextField()
    created_on = models.DateTimeField(auto_now_add=True)
    author = models.ForeignKey(User, on_delete=models.CASCADE)
    category = models.ForeignKey(Category, on_delete=models.SET_NULL, null=True)

    def __str__(self):
        return self.title    
            
# NOTES TO DEVELOPER:       
# [NOTE]       
# [NOTE]   
# [NOTE]   
# [NOTE]    
"""
SIGNALS_EXAMPLE = """from django.db.models.signals import post_save, post_delete
from django.dispatch import receiver
from django.contrib.auth.models import User
from django.db import models

from .models import Post, Category

@receiver(post_save, sender=User)
def create_user_profile(sender, instance, created, **kwargs):
    \"""
    Create a user profile when a new User is created.
    \"""
    if created:
        # You can customize this logic based on your needs
        Profile.objects.create(user=instance)

@receiver(post_save, sender=Post)
def update_post_count(sender, instance, created, **kwargs):
    \"""
    Update the post count in the author's profile when a new post is created.
    \"""
    if created:
        author_profile = instance.author.profile
        author_profile.post_count += 1
        author_profile.save()

@receiver(post_delete, sender=Post)
def decrease_post_count(sender, instance, **kwargs):
    \"""
    Decrease the post count in the author's profile when a post is deleted.
    \"""
    author_profile = instance.author.profile
    author_profile.post_count -= 1
    author_profile.save()

@receiver(post_save, sender=Category)
def update_category_count(sender, instance, created, **kwargs):
    \"""
    Update the post count in the category when a new post is created.
    \"""
    if created:
        instance.post_count = instance.post_set.count()
        instance.save()

@receiver(post_delete, sender=Category)
def decrease_category_count(sender, instance, **kwargs):
    \"""
    Decrease the post count in the category when a post is deleted.
    \"""
    instance.post_count = instance.post_set.count()
    instance.save()
    
# NOTES TO DEVELOPER:       
# [NOTE]       
# [NOTE]   
# [NOTE]   
# [NOTE]    
"""